import {Character, Zone} from "@prisma/client";
import ZoneRepository from "./repositories/zone.repository";
import ZoneService from "./services/zone.service";

interface ILoadedZone {
    zone: Zone;
    characters: Character[];
}

class ZoneManager {
    private static instance: ZoneManager;
    private loadedZones: ILoadedZone[] = [];

    private constructor() {}

    // Singleton instance getter
    public static getInstance(): ZoneManager {
        if (!ZoneManager.instance) {
            ZoneManager.instance = new ZoneManager();
        }
        return ZoneManager.instance;
    }

    // Method to initialize zone loading
    public async boot() {
        const zoneRepository = new ZoneRepository();

        if (!await zoneRepository.getById(1)) {
            const zoneService = new ZoneService();
            await zoneService.createDemoZone();
        }

        const zones = await zoneRepository.getAll();

        for (const zone of zones) {
            this.loadZone(zone);
        }

        console.log('[✅] Zone manager loaded');
    }

    // Method to handle individual zone loading
    public loadZone(zone: Zone) {
        this.loadedZones.push({
            zone,
            characters: []
        });
        console.log(`[✅] Zone ID ${zone.id} loaded`);
    }

    // Method to handle individual zone unloading
    public unloadZone(zoneId: number) {
        this.loadedZones = this.loadedZones.filter((loadedZone) => {
            return loadedZone.zone.id !== zoneId;
        });
        console.log(`[❌] Zone ID ${zoneId} unloaded`);
    }

    // Getter for loaded zones
    public getLoadedZones(): Zone[] {
        return this.loadedZones.map((loadedZone) => {
            return loadedZone.zone;
        });
    }
}

export default ZoneManager;
